﻿using Celeste.Mod.Entities;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using System.Collections.Generic;

namespace Celeste.Mod.Mine.Entities
{
    [Tracked(false)]
    public class BaseJumpBar : Solid
    {
        public static ParticleType P_Activate;
        public static ParticleType P_Down;
        private static SpriteBank spriteBank;
        private Sprite sprite;
        private float barring = 0;
        private float Maxbarring = 14;
        private bool decreasing = false;
        private float lastPSpeedX = 0;
        public bool IsVertical = false;
        private Collider PlayerWalKDetect;

        public BaseJumpBar(Vector2 position, bool vertical)
            : base(position, 16, 3, true)
        {
            IsVertical = vertical;
            if (IsVertical == true)
            {
                PlayerWalKDetect = new Hitbox(9, 16, Position.X - 3, Position.Y);
            }
            else
            {
                PlayerWalKDetect = new Hitbox(16, 3, Position.X, Position.Y - 3);
            }
        }

        public BaseJumpBar(EntityData e, Vector2 position, bool vertical)
            : this(e.Position + position, vertical)

        {
        }

        public override void Update()
        {
            base.Update();
            Level level = Scene as Level;
            Player p = level.Entities.FindFirst<Player>();
            if (p == null)
            {
                return;
            }
            if (PlayerWalKDetect.Collide(p) == true && decreasing == false)
            {
                if (barring > Maxbarring || barring == Maxbarring)
                {
                    if (IsVertical)
                    {
                        for (int i = 0; i < 30; i++)
                        {
                            Vector2 tempPos = new Vector2(this.Position.X, this.Position.Y + Calc.Range(Calc.Random, 0, 17));
                            if (Calc.Random.Next(0, 100) < 50)
                            {
                                level.ParticlesBG.Emit(P_Activate, tempPos, Calc.DegToRad * 0);
                            }
                            else
                            {
                                level.ParticlesBG.Emit(P_Activate, tempPos, Calc.DegToRad * 180);
                            }
                        }

                        lastPSpeedX = p.Speed.Y;
                        p.ExplodeLaunch(new Vector2(this.X, p.Y), true);
                        p.Speed.X = (int)(p.Speed.X * 1.2F);
                        p.Speed.Y = (int)(lastPSpeedX * 1.2F);
                    }
                    else
                    {
                        for (int i = 0; i < 30; i++)
                        {
                            Vector2 tempPos = new Vector2(this.Position.X + Calc.Range(Calc.Random, 0, 17), this.Position.Y + 1);
                            level.ParticlesBG.Emit(P_Activate, tempPos, Calc.DegToRad * 270);
                        }

                        lastPSpeedX = p.Speed.X;
                        p.ExplodeLaunch(new Vector2(p.X, this.Y - 1), true);
                        p.Speed.Y = (int)(p.Speed.Y * 1.2F);
                        p.Speed.X = (int)(lastPSpeedX * 1.2F);
                    }
                    decreasing = true;

                    barring = Maxbarring;
                }
                else
                {
                    if (decreasing == false)
                    {
                        if (p.DashAttacking)
                        {
                            barring = Maxbarring;
                            this.Update();
                        }

                        barring += Engine.DeltaTime * 20;
                    }
                }
            }
            else
            {
                if (barring > 0)
                {
                    if (decreasing == true)
                    {
                        barring -= Engine.DeltaTime * 5;
                    }
                    else
                    {
                        barring -= Engine.DeltaTime * 20;
                    }
                }
                if (barring < 0 || barring == 0)
                {
                    decreasing = false;
                    barring = 0;
                }
            }
        }

        public override void Render()
        {
            Level level = Scene as Level;
            base.Render();
            if (barring != 0)
            {
                Rectangle r = new Rectangle((int)(this.X + 1 + (14 / 2) - (int)barring / 2), (int)(this.Y + 1), (int)barring, 1);
                if (decreasing == true)
                {
                    Vector2 tempPos = new Vector2(this.Position.X + Calc.Range(Calc.Random, 0, 17), this.Position.Y + 1);
                    level.ParticlesBG.Emit(P_Down, tempPos, Calc.DegToRad * 270);

                    Draw.Rect(r, Color.Red);
                }
                else
                {
                    Draw.Rect(r, Color.Green);
                }
            }
            PlayerWalKDetect.Render(level.Camera);
        }

        public override void Added(Scene scene)
        {
            base.Added(scene);
            sprite = spriteBank.Create("JumpBar");
            sprite.Play("idle");
            Add(sprite);
        }

        public static void LoadContent()
        {
            spriteBank = new SpriteBank(GFX.Game, "Graphics/Mine/Sprites.xml");
        }
    }

    [CustomEntity("Mine/JumpBar")]
    public class JumpBar : BaseJumpBar
    {
        public JumpBar(Vector2 position)
             : base(position, false)

        {
        }

        public JumpBar(EntityData e, Vector2 position)
            : base(e, position, false)

        {
        }
    }

    [CustomEntity("Mine/VJumpBar")]
    public class VJumpBar : BaseJumpBar
    {
        public VJumpBar(Vector2 position)
             : base(position, true)

        {
            IsVertical = true;
        }

        public VJumpBar(EntityData e, Vector2 position)
           : base(e, position, true)

        {
        }
    }
}