﻿using Celeste;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using System.Collections;

using System.Collections.Generic;
using Celeste.Mod.Entities;

using On.Celeste;
using System.Linq;
using Steamworks;

namespace Celeste.Mod.Mine.Entities
{
    public class BetterCutscene : Entity
    {
        public Player player;
        public int width = 0;
        public int height = 0;
        public bool WasSkipped;
        public static bool canDo = true;
        public bool alreadyWatch = false;
        public bool RemoveOnSkipped = true;
        public bool badelineIsOut = false;
        public BadelineDummy badeline;
        public bool EndingChapterAfter;
        public Coroutine cinCoroutine;
        public Level Level;
        public string ParentName = "nullParent";
        public bool EndLevel = false;

        private bool Repeat = false;

        public BetterCutscene()

        {
            this.Visible = false;
        }

        public BetterCutscene(Vector2 position)
          : this()

        {
            this.Position = position;
        }

        public BetterCutscene(EntityData e, Vector2 position, string parentName, bool EndChapter = false, bool repeat = false)
          : this(e.Position + position)

        {
            Repeat = repeat;
            ParentName = parentName;

            width = e.Width;
            height = e.Height;
            this.Collider = new Hitbox(width, height);
            EndLevel = EndChapter;
        }

        public void LockPlayer()
        {
            player.StateMachine.State = 11;
            player.StateMachine.Locked = true;
        }

        public void UnlockPlayer()
        {
            player.StateMachine.Locked = false;
            player.StateMachine.State = 1;
        }

        public void UnlockPlayer(Player p)
        {
            p.StateMachine.Locked = false;
            p.StateMachine.State = 1;
        }

        public override void Added(Scene scene)
        {
            base.Added(scene);
            Level = (scene as Level);
        }

        public void Start()
        {
            Level.StartCutscene(SkipCutscene, true, EndLevel);
            OnBegin(Level);
        }

        public override void Update()
        {
            Level level = Engine.Scene as Level;
            if (level.Entities.FindFirst<Player>() != null)
            {
                if (this.CollideCheck(level.Entities.FindFirst<Player>()) && alreadyWatch == false && !level.Session.GetFlag("cutsceneAlreadyView" + ParentName))
                {
                    Start();
                    alreadyWatch = true;
                    canDo = false;
                }
            }

            if (Level.RetryPlayerCorpse != null)
            {
                Active = false;
            }
            else
            {
                base.Update();
            }
        }

        public void SkipCutscene(Level level)
        {
            WasSkipped = true;
            EndCutscene(level, true);
        }

        public void EndCutscene(Level level, bool removeSelf = true)
        {
            OnEnd(level);
            UnlockPlayer(level.Entities.FindFirst<Player>());
            canDo = false;

            if (badelineIsOut == true)
            {
                badeline.RemoveSelf();
            }
            level.EndCutscene();
            if (Repeat == false)
            {
                level.Session.SetFlag("cutsceneAlreadyView" + ParentName, true); ; // Sets flag to not load
            }
            RemoveSelf();
        }

        public static IEnumerator CameraTo(Vector2 target, float duration, Ease.Easer ease, float delay = 0f)
        {
            Level level = Engine.Scene as Level;
            Vector2 from = level.Camera.Position;
            if (ease == null)
            {
                ease = Ease.CubeInOut;
            }
            if (delay > 0f)
            {
                yield return delay;
            }

            for (float p = 0f; p < 1f; p += Engine.DeltaTime / duration)
            {
                level.Camera.Position = from + (target - from) * ease(p);
                yield return 0;
            }
            level.Camera.Position = target;
        }

        public static IEnumerator CameraTo(Vector2 target, float duration)
        {
            return CameraTo(target, duration, null, 0f);
        }

        public void OnBegin(Level level)
        {
            this.player = level.Entities.FindFirst<Player>();
            cinCoroutine = new Coroutine(Routine(), true);
            Add(cinCoroutine);
        }

        public void OnEnd(Level level)
        {
        }

        public IEnumerator walkPlayer(int X)
        {
            yield return player.DummyWalkTo(X);
        }

        public IEnumerator Duck(float delay)
        {
            player.Ducking = true;
            yield return delay;
        }

        /// <summary>
        /// thank to DJMAPHELPER
        /// </summary>
        public IEnumerator BadelineAppears()
        {
            badelineIsOut = true;
            Audio.Play("event:/char/badeline/maddy_split", player.Position);
            Level.Add(badeline = new BadelineDummy(player.Center));
            Level.Displacement.AddBurst(badeline.Center, 0.5f, 8f, 32f, 0.5f);
            Level.Session.Inventory.Dashes = 1;
            player.Dashes = 1;
            badeline.Sprite.Scale.X = -1f;
            yield return badeline.FloatTo(player.Center + new Vector2(18f, -10f), -1, faceDirection: false);
            yield return 0.2f;
            player.Facing = Facings.Right;
            yield return null;
        }

        public IEnumerator BadelineAppears(Vector2 customPosOffset)
        {
            badelineIsOut = true;
            Audio.Play("event:/char/badeline/maddy_split", player.Position);
            Level.Add(badeline = new BadelineDummy(player.Center));
            Level.Displacement.AddBurst(badeline.Center, 0.5f, 8f, 32f, 0.5f);

            Level.Session.Inventory.Dashes = 1;

            player.Dashes = 1;
            badeline.Sprite.Scale.X = -1f;
            yield return badeline.FloatTo(player.Center + customPosOffset, -1, faceDirection: false);
            yield return 0.2f;
            player.Facing = Facings.Right;
            yield return null;
        }

        public IEnumerator BadelineRejoin(bool has2dashAfter = false)
        {
            badelineIsOut = false;
            if (has2dashAfter == true)
            {
                Level.Session.Inventory.Dashes = 2;
                player.Dashes = 2;
            }

            Audio.Play("event:/new_content/char/badeline/maddy_join_quick", badeline.Position);
            Vector2 from = badeline.Position;
            for (float p = 0f; (double)p < 1.0; p += Engine.DeltaTime / 0.25f)
            {
                badeline.Position = Vector2.Lerp(from, player.Position, Ease.CubeIn(p));
                yield return null;
            }
            Level.Displacement.AddBurst(player.Center, 0.5f, 8f, 32f, 0.5f);
            badeline.RemoveSelf();
        }

        /// <summary>
        /// Just a sample
        /// </summary>
        public virtual IEnumerator Routine()
        {
            LockPlayer();

            yield return Textbox.Say("theFilePath");
            yield return 0.5f;
            yield return BadelineAppears();
            yield return 0.3f;

            yield return Textbox.Say("theFilePath");
            yield return BadelineRejoin();
            UnlockPlayer();
            EndCutscene(Level);
        }
    }
}