﻿using Celeste.Mod.Entities;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Celeste.Mod.Mine.Entities
{
    public class CrackedStoneRenderManager : Entity
    {
        private List<CrackedStone> allStones;
        private List<Sprite> allSprite;

        public CrackedStoneRenderManager(Vector2 position, Scene scene)
           : base(position)
        {
            allStones = new List<CrackedStone>();
            foreach (CrackedStone item in scene.Entities.FindAll<CrackedStone>())
            {
                if (item != null)
                {
                    allStones.Add(item);
                }
            }
        }

        public override void Update()
        {
            base.Update();
        }

        public override void Render()
        {
            base.Render();
            foreach (CrackedStone item in allStones)
            {
                if (item != null && item.removed == false && item.ShouldRender == true)
                {
                    Draw.Rect(new Rectangle((int)item.X - 1, (int)item.Y - 1, (int)item.Width + 2, (int)item.Height + 2), Color.Black);
                }
            }
        }
    }

    [Tracked(false)]
    [CustomEntity("Mine/CrackedStone")]
    public class CrackedStone : Solid
    {
        private static SpriteBank spriteBank;
        public Sprite sprite;
        public bool removed = false;
        private int spriteFrame = 0;
        public bool ShouldRender = false;

        public CrackedStone(Vector2 position)
            : base(position, 8, 8, true)
        {
        }

        public CrackedStone(EntityData e, Vector2 position)
            : this(e.Position + position)

        {
        }

        public void removeBlock()
        {
            removed = true;
            this.RemoveSelf();
        }

        public override void Update()
        {
            Level l = Scene as Level;

            ShouldRender = l.IsInCamera(this.Position, 8);
            base.Update();
            if (Scene.Entities.FindFirst<CrackedStoneRenderManager>() == null)
            {
                Scene.Entities.Add(new CrackedStoneRenderManager(new Vector2(0, 0), Scene));
            }
            // sprite.SetAnimationFrame(spriteFrame);
        }

        public override void Render()
        {
            //this.Depth = 0;
            base.Render();
            // Draw.Rect(new Rectangle((int)this.X - 1, (int)this.Y - 1, (int)this.Width + 2,
            // (int)this.Height + 2), Color.Red); base.Render();
        }

        public override void Added(Scene scene)

        {
            base.Added(scene);
            this.Depth = -100;
            sprite = spriteBank.Create("CrackedStone");
            Random rnd = new Random((int)(this.Position.X * this.Position.Y));
            sprite.Play(rnd.Range(1, 6).ToString());
            Logger.Log("hey", rnd.Range(1, 6).ToString());
            // spriteFrame = sprite.CurrentAnimationFrame; sprite.Rotation = ((Calc.Random.Range(0,
            // 4) * 90f).ToRad());
            if (rnd.Range(0, 100) > 50)
            {
                sprite.FlipX = true;
            }
            rnd = new Random((int)(this.Position.X + this.Position.Y));
            if (rnd.Range(0, 100) > 50)
            {
                sprite.FlipY = true;
            }
            Add(sprite);
        }

        public static void LoadContent()
        {
            spriteBank = new SpriteBank(GFX.Game, "Graphics/Mine/Sprites.xml");
        }
    }
}