﻿using Celeste.Mod.Entities;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using System.Collections.Generic;
using System.Runtime.Remoting.Lifetime;

namespace Celeste.Mod.Mine.Entities
{
    [Tracked(false)]
    [CustomEntity("Mine/CrystalShooter")]
    public class CrystalShooter : Entity
    {
        private static SpriteBank spriteBank;
        private Sprite sprite;
        private const float shootDelay = 350;
        private float shooting = 350;
        private Vector2 offsetPlayerPos = new Vector2(0, 0);
        private VertexLight light;

        public CrystalShooter(Vector2 position)
            : base(position)
        {
            Collider = new Hitbox(16, 16);
        }

        public CrystalShooter(EntityData e, Vector2 position)
            : this(e.Position + position)

        {
        }

        public override void Update()
        {
            Level level = Scene as Level;
            base.Update();
            Player player = level.Tracker.GetEntity<Player>();
            if (player == null || player.JustRespawned || !level.IsInCamera(Position, 8f))
            {
                return;
            }
            shooting -= 1;
            if (shooting == 180)
            {
                sprite.Play("charge");
                light.Color = Color.Purple;
            }
            if (shooting <= 180)
            {
                if (shooting % 54 == 1)
                {
                    if (player != null)
                    {
                        level.Add(Engine.Pooler.Create<CrystalBall>().Init(this, new
                        Vector2(player.Position.X + player.Width / 2, player.Position.Y +
                        player.Height /
                        2) + offsetPlayerPos * 3));
                    }
                }
            }
            if (shooting == 0)
            {
                light.Color = Color.White;
                shooting = shootDelay;
                sprite.Play("base");
            }
            offsetPlayerPos = player.Position - player.PreviousPosition;
        }

        public override void Render()
        {
            base.Render();
        }

        public override void Added(Scene scene)
        {
            base.Added(scene);
            sprite = spriteBank.Create("CrystalShooter");
            sprite.Play("base");
            Add(sprite);

            Add(new BloomPoint(0.8f, 16f));
            Add(light = new VertexLight(Color.White, 1f, 16, 48));
        }

        public static void LoadContent()
        {
            spriteBank = new SpriteBank(GFX.Game, "Graphics/Mine/Sprites.xml");
        }
    }
}