﻿using Celeste.Mod.Entities;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using System.Collections;
using System.Collections.Generic;

namespace Celeste.Mod.Mine.Entities
{
    [Tracked(false)]
    [CustomEntity("Mine/MagicLight")]
    public class MagicLight : Entity
    {
        private SineWave sine;
        private SineWave sinex;
        private static SpriteBank spriteBank;
        private Sprite sprite;
        private Wiggler wiggler;
        private VertexLight lightcomponents;
        private BloomPoint BloomPoint;
        private float flying = 0.0f;
        private double time;
        private int timeMult = 1;

        public MagicLight(Vector2 position)

            : base(position)
        {
            //Position = position + Vector2.UnitX * 4f;
            //Length = Math.Max(16, length);
            //base.Depth = 2000;
            //Sprite image = sprite;
            //for (int i = 0; i < Length - 8; i += 8)
            //{
            //    Add(image = mTexture);
            //    image.Origin.X = 4f;
            //    image.Origin.Y = -i;
            //    images.Add(image);
            //}
            //Add(image = mTexture);
            //image.Origin.X = 4f;
            //Add(image = mTexture);
            //image.Origin.X = 4f;
            //image.Origin.Y = -(Length - 8);
            //images.Add(image);
            //Add(bloom = new BloomPoint(Vector2.UnitY * (Length - 4), 1f, 48f));
            //Add(light = new VertexLight(Vector2.UnitY * (Length - 4), Color.White, 1f, 24, 48));
        }

        public MagicLight(EntityData e, Vector2 position)
            : this(e.Position + position)

        {
        }

        public override void Update()
        {
            sprite.Y = -(sprite.Height / 2) + sine.Value * 1;
            sprite.X = -(sprite.Width / 2) + sinex.Value * 1;
            // sine.Update();
            base.Update();
        }

        public override void Render()
        {
            base.Render();
        }

        public override void Added(Scene scene)
        {
            time = 1;
            this.Add(lightcomponents = new VertexLight(Color.Yellow, 1, 18, 20));
            // Add(wiggler = Wiggler.Create(1f, 4f));

            base.Added(scene);
            sprite = spriteBank.Create("magicLight");
            sprite.Play("idle");
            sprite.X = -(sprite.Width / 2);
            sprite.Y = -(sprite.Height / 2);

            // sprite.JustifyOrigin(-sprite.Center.X, -sprite.Center.Y); sprite.CenterOrigin();
            Add(sprite);
            Add(BloomPoint = new BloomPoint(0.5f, 17));
            Add(sine = new SineWave(0.6f));
            Add(sinex = new SineWave(0.6f));
            sine.Randomize();
            sinex.Randomize();
        }

        public static void LoadContent()
        {
            spriteBank = new SpriteBank(GFX.Game, "Graphics/Mine/Sprites.xml");
        }
    }

    [CustomEntity("Mine/MoveMagicLight")]
    public class MoveMagicLight : Entity
    {
        private SineWave sine;
        private SineWave sinex;
        private static SpriteBank spriteBank;
        private Sprite sprite;
        private VertexLight lightcomponents;
        private Vector2 targetPosition;
        private Vector2 BasePosition;
        private float lerping = 1;
        private bool increase = false;
        private BloomPoint BloomPoint;

        public MoveMagicLight(Vector2 position)
            : base(position)
        {
        }

        public MoveMagicLight(Vector2 position, Vector2 target)
            : base(position)
        {
            BasePosition = position;
            targetPosition = target;
            lightcomponents = new VertexLight(Color.Green, 1, 25, 30);
            this.Add(lightcomponents);
            Add(BloomPoint = new BloomPoint(0.5f, 25));
        }

        public MoveMagicLight(EntityData data, Vector2 offset)
        : this(data.Position + offset, data.Nodes[0] + offset)
        {
        }

        public override void Update()
        {
            sprite.Y = -(sprite.Height / 2) + sine.Value * 1;
            sprite.X = -(sprite.Width / 2) + sinex.Value * 1;
            Position = Vector2.Lerp(BasePosition, targetPosition, Ease.SineInOut(lerping));
            if (increase == true)
            {
                lerping += Engine.DeltaTime;
            }
            else
            {
                lerping -= Engine.DeltaTime;
            }
            if (lerping > 1 && increase == true)
            {
                increase = false;
                lerping = 0.99f;
            }
            else if (lerping < 0 && increase == false)
            {
                lerping = 0.01f;
                increase = true;
            }
            base.Update();
        }

        public override void Render()
        {
            base.Render();
        }

        public override void Added(Scene scene)
        {
            Add(sine = new SineWave(0.6f));
            Add(sinex = new SineWave(0.6f));
            // Add(wiggler = Wiggler.Create(1f, 4f));

            base.Added(scene);
            sprite = spriteBank.Create("magicLightMove");
            sprite.Play("idle");
            sprite.X = -(sprite.Width / 2);
            sprite.Y = -(sprite.Height / 2);
            // sprite.JustifyOrigin(-sprite.Center.X, -sprite.Center.Y); sprite.CenterOrigin();
            Add(sprite);
        }

        public static void LoadContent()
        {
            spriteBank = new SpriteBank(GFX.Game, "Graphics/Mine/Sprites.xml");
        }
    }
}