﻿using Celeste;
using Microsoft.Xna.Framework;
using Monocle;
using System.Collections.Generic;
using Celeste.Mod.Entities;
using System;
using On.Celeste;
using System.Linq;

namespace Celeste.Mod.Mine.Entities
{
    [Tracked(false)]
    [CustomEntity("Mine/DashZone")]
    public class DashZone : Entity
    {
        private float DZHeight = 0;
        private float DZWidth = 0;
        private bool playerEntered = false;
        public static ParticleType P_Zone;
        public static ParticleType P_Player;
        private Vector2 initialPosition;
        public Vector2 PlayerSpeedInside;
        private Vector2 previousPlayerSpeed;
        public bool previousVisible = true;
        public Collider previousHit;
        private List<Vector2> particles = new List<Vector2>();
        private SoundSource soundSource;
        public Vector2 particleDir;
        public float particleDown = 1;

        private float[] speeds = new float[3]
        {
       10f,
      20f,
      30f
        };

        public DashZone(Vector2 position, float width, float height)
            : base(position)
        {
            this.Collider = new Hitbox(width, height);
            DZWidth = width;
            DZHeight = height;
            for (int i = 0; (float)i < width * height / 32f; i++)
            {
                particles.Add(new Vector2(Calc.Random.NextFloat(width - 1f), Calc.Random.NextFloat(height - 1f)));
            }
            Add(soundSource = new SoundSource());
        }

        public DashZone(EntityData data, Vector2 offset)
            : this(data.Position + offset, data.Width, data.Height)
        {
        }

        private void OnRenderBloom()
        {
            for (int i = 0; i < 3; i++)
            {
                Rectangle rrectangle = new Rectangle((int)this.X - i, (int)this.Y - i, (int)DZWidth + i * 2, (int)DZHeight + i * 2);
                Draw.Rect(rrectangle, Color.FromNonPremultiplied(255, 255, 255, (int)((100 - particleDown * 100) / i)));
            }
            // Rectangle rectangle = new Rectangle((int)this.X, (int)this.Y, (int)DZWidth,
            // (int)DZHeight); Draw.Rect(rectangle, Color.FromNonPremultiplied(255, 255, 255, 100));
        }

        public override void Added(Scene scene)
        {
            base.Added(scene);
            CustomBloom bloomPoint = new CustomBloom(OnRenderBloom);
            this.Add(bloomPoint);
        }

        public override void Removed(Scene scene)
        {
            base.Removed(scene);
        }

        public Vector2 getFinalSpeed(Vector2 m)
        {
            if (m.X < 0)
            {
                m.X = Calc.Max(-0.9f, m.X);
            }
            else
            {
                m.X = Calc.Min(0.9f, m.X);
            }
            if (m.Y < 0)
            {
                m.Y = Calc.Max(-0.9f, m.Y);
            }
            else
            {
                m.Y = Calc.Min(0.9f, m.Y);
            }
            if (Math.Abs(m.X) + Math.Abs(m.Y) > 1.8f)
            {
                if (m.X > 0.9)
                {
                    m.X = 0.9f;
                }
                if (m.X < -0.9f)
                {
                    m.X = -0.9f;
                }
                if (m.Y > 0.9)
                {
                    m.Y = 0.9f;
                }
                if (m.Y < -0.9f)
                {
                    m.Y = -0.9f;
                }
            }
            return m;
        }

        public override void Update()
        {
            float height = base.Height;
            float width = base.Width;
            int i = 0;
            int num = speeds.Length;

            Level level = Scene as Level;
            if (level == null)
            {
                return;
            }
            // level.ParticlesFG.Emit(P_Zone, 5, new Vector2(this.Position.X + DZWidth / 2,
            // this.Position.Y + DZHeight / 2), new Vector2(DZWidth / 2, DZHeight / 2),
            // Calc.DegToRad * 90);
            Player p = level.Entities.FindFirst<Player>();

            if (p != null)
            {
                if (this.CollideCheck(p))
                {
                    if (particleDown > 0)
                    {
                        particleDown -= 0.02f;
                    }
                    if (particleDown < 0)
                    {
                        particleDown = 0;
                    }
                }
                if (this.CollideCheck(p))
                {
                    p.StateMachine.State = 3; // 3
                    Vector2 m = getFinalSpeed(Input.Aim.Value);

                    Vector2 aim = m * 200;
                    if (playerEntered == false)
                    {
                        soundSource.Play("event:/game/06_reflection/feather_state_loop", "feather_speed", 0.1f);

                        previousVisible = p.Visible;
                        previousPlayerSpeed = p.Speed;
                        for (int pi = 0; pi < 20; pi++)
                        {
                            Vector2 aimp = Input.Aim.Value * pi * 20;
                            level.ParticlesBG.Emit(P_Player, p.Center, (aimp.Angle().ToDeg() + 180).ToRad());
                        }
                    }

                    particleDir = aim / 100;
                    particleDir.Y += particleDown;
                    //if (Calc.Abs(aim).Length() < 1)
                    //{
                    //    particleDir.Y += 1;
                    //}
                    playerEntered = true;
                    if (p.DashAttacking)
                    {
                        level.ParticlesBG.Emit(P_Player, p.Center, Color.Cyan, aim.Angle());
                    }
                    else
                    {
                        level.ParticlesBG.Emit(P_Player, p.Center, aim.Angle());
                    }
                    p.Speed = new Vector2(((int)aim.X), (int)(aim.Y));
                    PlayerSpeedInside = p.Speed;
                    p.DummyGravity = false;
                    Input.Rumble(RumbleStrength.Light, RumbleLength.Short);
                }
                else
                {
                    PlayerSpeedInside *= 0.7f;
                    particleDir.X = 0;
                    if (particleDown < 1)
                    {
                        particleDown += 0.02f;
                    }
                    particleDir.Y = particleDown;
                    if (playerEntered == true)
                    {
                        soundSource.Stop(true);
                        // p.ExplodeLaunch((Input.Aim.Value) * -2, true);
                        p.StateMachine.State = 0; // 3
                                                  // p.Visible = previousVisible; // (Input.Aim.Value);
                        if (p.DashAttacking)
                        {
                            for (int pi = 0; pi < 20; pi++)
                            {
                                Vector2 aim = Input.Aim.Value * (pi * 20);
                                level.ParticlesBG.Emit(P_Player, p.Center, (aim.Angle().ToDeg()).ToRad());
                            }
                            Vector2 m = getFinalSpeed(Input.Aim.Value);

                            p.Speed += (m) * 200;
                            p.Dashes = 1;
                        }
                        else
                        {
                            for (int pi = 0; pi < 30; pi++)
                            {
                                Vector2 aim = Input.Aim.Value * (pi * 20);
                                level.ParticlesBG.Emit(P_Player, p.Center, (aim.Angle().ToDeg() + 180).ToRad());
                            }
                            p.Dashes = 1;
                            Vector2 m = getFinalSpeed(Input.Aim.Value);
                            p.Speed += m * 40;
                        }
                    }
                    playerEntered = false;
                    // p.Select
                }
            }

            for (int count = particles.Count; i < count; i++)
            {
                Vector2 value = particles[i] + particleDir * speeds[i % num] * Engine.DeltaTime;

                value.Y %= height - 1f;
                value.X %= width - 1f;
                particles[i] = value;
                if (!CollidePoint(particles[i] + Position))
                {
                    particles[i] = new Vector2(Calc.Random.NextFloat(width - 1f), Calc.Random.NextFloat(height - 1f));
                }
            }
            base.Update();
        }

        public override void Render()
        {
            Level level = Scene as Level;
            Player p = level.Tracker.GetEntity<Player>();
            Color color = Color.White * 0.5f;
            foreach (Vector2 particle in particles)
            {
                Draw.Pixel.Draw(Position + particle, Vector2.Zero, color);
            }
            // Draw.Text(Draw.DefaultFont, Input.Aim.Value.X.ToString(), this.Position, Color.White);
            Draw.Rect(this.Collider, Color.FromNonPremultiplied(255, 255, 255, 75));
            Draw.HollowRect(this.Collider, Color.FromNonPremultiplied(255, 255, 255, 255));
            //Draw.Rect(this.Position, DZWidth, DZHeight, new Color(1f, 1f, 1f, 0f));
            // Draw.HollowRect(this.Position, DZWidth, DZHeight, new Color(1f, 1f, 1f, 1f));
        }
    }
}