﻿using Celeste.Mod.Entities;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Celeste.Mod.Mine.Entities
{
    [Tracked(false)]
    /// BIG THANK to @max480#4596 on the celeste discord server he help me a lot !
    public class LazerBeam : Monocle.Entity
    {
        public enum Directions
        {
            Up = 3,
            Down = 1,
            Left = 0,
            Right = 2
        }

        private VertexLight lighting;
        private static SpriteBank spriteBank;
        private Sprite sprite;
        private BloomPoint bloom;
        private Player player;
        public int maxRange = 2048;
        public Vector2 direction;
        public Vector2 deathDirection; // for later
        public Directions directionLook = Directions.Up; // 0 = UP, 1 = RIGHT, 2 = DOWN, 3 = LEFT
        private int lazerRange = 0;
        public Particle lazerp;
        public ParticleType lazerpt;
        public bool particleInitialised = false;
        public Entity d;
        private Rectangle r;
        public Collider MaxCollide;

        public void initParticle()
        {
            Level level = Scene as Level;

            lazerpt = new ParticleType();

            lazerpt.Acceleration = direction * -1;
            lazerpt.Color = new Color(255, 0, 0, 255);
            lazerpt.Color2 = new Color(255, 0, 0, 0);
            lazerpt.ColorMode = ParticleType.ColorModes.Fade;
            lazerpt.Direction = 0f;
            lazerpt.DirectionRange = 0f;
            lazerpt.FadeMode = ParticleType.FadeModes.Linear;
            lazerpt.Friction = 1f;
            lazerpt.LifeMax = 7;
            lazerpt.LifeMin = 1;
            lazerpt.Size = 2;
            lazerpt.SizeRange = 4;
            lazerpt.SpeedMax = 10;
            lazerpt.SpeedMin = 1;
            lazerpt.SpeedMultiplier = 1.1F;
            //sysp = new ParticleSystem(1, 1024);

            //lazerParticleEmmiter = new ParticleEmitter(sysp, lazerp, this, new Vector2((this.Position.X) + this.Width / 2, this.Position.Y + lazerRange), new Vector2((this.Position.X) + this.Width / 2, this.Position.Y + lazerRange), 10, 1, 1);
            //lazerParticleEmmiter.Amount = 10;
            //lazerParticleEmmiter.Direction = 1;
            //lazerParticleEmmiter.Interval = 1;
            //lazerParticleEmmiter.Position = this.Position;
            //lazerParticleEmmiter.Visible = true;
            //lazerParticleEmmiter.Active = true;
            //lazerParticleEmmiter.HandleGraphicsCreate();
            //lazerParticleEmmiter.EntityAdded(Scene);
            r = new Rectangle();
        }

        private void LoadHitbox()
        {
            if (directionLook == Directions.Up)
            {
                Collider = new Hitbox(16, maxRange, 0, -maxRange);
            }
            else if (directionLook == Directions.Down)
            {
                Collider = new Hitbox(16, maxRange);
            }
            else if (directionLook == Directions.Right)
            {
                Collider = new Hitbox(maxRange, 16);
            }
            else if (directionLook == Directions.Left)
            {
                Collider = new Hitbox(maxRange, 16, -maxRange, 0);
            }
        }

        public LazerBeam(Vector2 position, Directions directions)

           : base(position)
        {
            Collidable = true;
            deathDirection = new Vector2(0, 0);
            directionLook = directions;
            switch (directionLook)
            {
                case Directions.Up:
                    direction = new Vector2(0, -1);
                    break;

                case Directions.Down:
                    direction = new Vector2(0, 1);
                    break;

                case Directions.Left:
                    direction = new Vector2(-1, 0);
                    break;

                case Directions.Right:
                    direction = new Vector2(1, 0);
                    break;

                default:
                    direction = new Vector2(0, 0);
                    break;
            }

            LoadHitbox();
            MaxCollide = this.Collider;
        }

        // heavily based on Pandora's Box
        public void AdjustHitboxToNearestSolidTile(Grid grid)
        {
            int offset = maxRange / 2;
            SetLazerHitboxLength(offset);

            while (offset >= 1)
            {
                bool colliding = grid.Collide(this);
                int sign = colliding ? -1 : 1;

                offset /= 2;
                SetLazerHitboxLength(maxRange + sign * offset);
            }

            if (grid.Collide(this))
            {
                SetLazerHitboxLength(maxRange - 1);
            }
        }

        public void SetLazerHitboxLength(int length)
        {
            if (directionLook == Directions.Up)
            {
                Collider.Position.Y = -length;
                Collider.Height = length;
            }
            else if (directionLook == Directions.Down)
            {
                Collider.Height = length;
            }
            else if (directionLook == Directions.Right)
            {
                Collider.Width = length;
            }
            else if (directionLook == Directions.Left)
            {
                Collider.Position.X = -length;
                Collider.Width = length;
            }
            maxRange = length;
        }

        public LazerBeam(EntityData data, Vector2 offset, Directions directions)
           : this(data.Position + offset, directions) { }

        public static Entity loadUp(Vector2 position)
        => new LazerBeam(position, Directions.Up);

        public static Entity loadDown(Vector2 position)
        => new LazerBeam(position, Directions.Down);

        public static Entity loadRight(Vector2 position)
        => new LazerBeam(position, Directions.Right);

        public static Entity loadLeft(Vector2 position)
        => new LazerBeam(position, Directions.Left);

        public override void Added(Scene scene)
        {
            sprite = spriteBank.Create("lazerBeam");
            sprite.Play("idle");
            Add(sprite);
            sprite.CenterOrigin();
            if (directionLook == Directions.Up)
            {
                sprite.Rotation = 3.14F;
                sprite.X = this.Width / 2;
            }
            else if (directionLook == Directions.Down)
            {
                sprite.Rotation = 0;
                sprite.X = this.Width / 2;
            }
            else if (directionLook == Directions.Right)
            {
                sprite.Rotation = (3 * 3.14F) / 2;
                sprite.Y = this.Height / 2;
            }
            else if (directionLook == Directions.Left)
            {
                sprite.Rotation = (3.14F) / 2;
                sprite.Y = this.Height / 2;
            }
            base.Added(scene);
            this.Collidable = true;
            bloom = new BloomPoint(0.5F, 50);
            bloom.Position = new Vector2(this.Width / 2, this.Height / 2);
            Add(bloom);
            Add(lighting = new VertexLight(sprite.Position, Color.Red, 0.75f, 20, 30));
        }

        public override void Render()
        {
            if (particleInitialised == false)
            {
                initParticle();
                particleInitialised = true;
            }
            Level level = Scene as Level;
            //Draw.Rect(this.Collider, Color.White);
            if (directionLook == Directions.Up)
            {
                //.Draw(new Vector2(this.X, this.Y - LazerTexture.Height), new Vector2(this.X + this.Width / 2, this.Y - LazerTexture.Height / 2), Color.White, new Vector2(1, 1), (int)directionLook * 90);
                Draw.Rect((this.Position.X), this.Position.Y, this.Width, lazerRange, new Color(255, 0, 0, 255));
            }
            else if (directionLook == Directions.Down)
            {
                Draw.Rect((this.Position.X), this.Position.Y, this.Width, lazerRange, new Color(255, 0, 0, 255));
            }
            else if (directionLook == Directions.Right)
            {
                Draw.Rect((this.Position.X), this.Position.Y, lazerRange, this.Height, new Color(255, 0, 0, 255));
            }
            else if (directionLook == Directions.Left)
            {
                Draw.Rect((this.Position.X), this.Position.Y, lazerRange, this.Height, new Color(255, 0, 0, 255));
            }

            base.Render();
        }

        public void UpdateLazer()
        {
            Level level = Scene as Level;
            List<int> values;
            switch (directionLook)
            {
                case Directions.Down:
                    values = CollideAll<Solid>().Select(value => (int)value.Top).ToList();
                    lazerRange = values.Count > 0 ? (int)(values.Min() - Top - 1) : maxRange;
                    break;

                case Directions.Up:
                    values = CollideAll<Solid>().Select(value => (int)value.Bottom).ToList();
                    lazerRange = values.Count > 0 ? (int)(values.Max() - Bottom + 1) : -maxRange;
                    break;

                case Directions.Left:
                    values = CollideAll<Solid>().Select(value => (int)value.Right).ToList();
                    lazerRange = values.Count > 0 ? (int)(values.Max() - Right - 1) : -maxRange;
                    break;

                default: // visual studio error

                    values = CollideAll<Solid>().Select(value => (int)value.Left).ToList();
                    lazerRange = values.Count > 0 ? (int)(values.Min() - Left + 1) : maxRange;
                    break;
            }
        }

        public override void Update()
        {
            base.Update();
            if (particleInitialised == false)
            {
                initParticle();
                particleInitialised = true;
            } // i should optimise this

            UpdateLazer();
            if (directionLook == Directions.Up)
            {
                //.Draw(new Vector2(this.X, this.Y - LazerTexture.Height), new Vector2(this.X + this.Width / 2, this.Y - LazerTexture.Height / 2), Color.White, new Vector2(1, 1), (int)directionLook * 90);
                r = new Rectangle((int)(this.Position.X), (int)this.Position.Y + lazerRange, (int)this.Width, (int)-lazerRange);
            }
            else if (directionLook == Directions.Down)
            {
                r = new Rectangle((int)(this.Position.X), (int)this.Position.Y, (int)this.Width, (int)lazerRange);
            }
            else if (directionLook == Directions.Right)
            {
                r = new Rectangle((int)(this.Position.X), (int)this.Position.Y, lazerRange, (int)this.Height);
            }
            else if (directionLook == Directions.Left)
            {
                r = new Rectangle((int)(this.Position.X) + lazerRange, (int)this.Position.Y, -lazerRange, (int)this.Height);
            }
            Random rnd = new Random();
            Level level = Scene as Level;
            level.ParticlesFG.Emit(lazerpt, (int)(Math.Abs(lazerRange) * 0.001) + 1, new Vector2(r.X + r.Width / 2, r.Y + r.Height / 2), new Vector2(r.Width / 2 + 2, r.Height / 2 + 2), direction.Angle());
            var player = level.Entities.FindFirst<Player>();
            if (player != null)
            {
                if (player.CollideRect(r))
                {
                    player.Die(direction);
                }
            }
            var tnts = level.Entities.FindAll<TNT>();
            if (tnts.Count != 0)
            {
                foreach (TNT item in tnts)
                {
                    if (item.CollideRect(r) && item.exploding == false)
                    {
                        item.exploding = true;
                        item.explosionCoolDown = -1;
                    }
                }
            }
        }

        public override void Awake(Scene scene)
        {
            base.Awake(scene);

            SolidTiles tiles = CollideFirst<SolidTiles>();
            if (tiles != null)
            {
                AdjustHitboxToNearestSolidTile(tiles.Collider as Grid);
            }
        }

        public void Reset()
        {
            LoadHitbox();
            SolidTiles tiles = CollideFirst<SolidTiles>();
            if (tiles != null)
            {
                AdjustHitboxToNearestSolidTile(tiles.Collider as Grid);
            }
        }

        public static void LoadContent()
        {
            spriteBank = new SpriteBank(GFX.Game, "Graphics/Mine/Sprites.xml");
        }

        public static void Unload()
        {
        }
    }

    //[CustomEntity(
    //    "Mine/LazerBeamUp = loadUp",
    //    "Mine/LazerBeamDown = loadDown",
    //    "Mine/LazerBeamLeft = loadLeft",
    //    "Mine/LazerBeamRight = loadRight"
    //    )]
    [CustomEntity("Mine/LazerBeamUp")]
    public class LazerBeamUp : LazerBeam
    {
        public LazerBeamUp(Vector2 pos)
           : base(pos, Directions.Up) { }

        public LazerBeamUp(EntityData d, Vector2 offset)
          : base(d, offset, Directions.Up) { }
    }

    [CustomEntity("Mine/LazerBeamDown")]
    public class LazerBeamDown : LazerBeam
    {
        public LazerBeamDown(Vector2 pos)
           : base(pos, Directions.Down) { }

        public LazerBeamDown(EntityData d, Vector2 offset)
          : base(d, offset, Directions.Down) { }
    }

    [CustomEntity("Mine/LazerBeamLeft")]
    public class LazerBeamLeft : LazerBeam
    {
        public LazerBeamLeft(Vector2 pos)
           : base(pos, Directions.Left) { }

        public LazerBeamLeft(EntityData d, Vector2 offset)
          : base(d, offset, Directions.Left) { }
    }

    [CustomEntity("Mine/LazerBeamRight")]
    public class LazerBeamRight : LazerBeam
    {
        public LazerBeamRight(Vector2 pos)
           : base(pos, Directions.Right) { }

        public LazerBeamRight(EntityData d, Vector2 offset)
          : base(d, offset, Directions.Right) { }
    }
}