﻿using Celeste.Mod.Entities;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Celeste.Mod.Mine.Entities
{
    [Tracked(false)]
    [CustomEntity("Mine/Minecart")]
    public class Minecart : Entity
    {
        private static SpriteBank spriteBank;
        private Sprite sprite;
        private Vector2 velocity;

        public Minecart(Vector2 position)
            : base(position)
        {
            this.Collider = new Hitbox(10, 11);
        }

        public Minecart(EntityData e, Vector2 position)
            : this(e.Position + position)

        {
        }

        public override void Update()
        {
            base.Update();
            Level lev = Scene as Level;
            var collideAll = CollideAll<Rail>();
            if (collideAll.Count == 0 || collideAll[0] == null)
            {
                var collideAll2 = CollideAll<Solid>();
                if (collideAll2.Count == 0 || collideAll2[0] == null)
                {
                    velocity.Y += 10;
                }
            }
            else
            {
                velocity.Y = 0;
                Player player = lev.Entities.FindFirst<Player>();
                if (player == null)
                {
                    return;
                }
                if (CollideCheck(player))
                {
                    if ((player.Position.X - player.PreviousPosition.X) < 0)
                    {
                        velocity.X -= 0.3F;
                    }
                    if ((player.Position.X - player.PreviousPosition.X) > 0)
                    {
                        velocity.X += 0.3F;
                    }
                }
                List<Minecart> Minecarts = lev.Entities.FindAll<Minecart>();

                foreach (Minecart v in Minecarts)
                {
                    if (v != this)
                    {
                        if (this.CollideCheck(v))
                        {
                            this.velocity.X += v.velocity.X / 2;
                            v.velocity.X /= 2;
                        }
                    }
                }
                if (collideAll[0].Top < this.Top)
                {
                    this.Y -= 1;
                }
            }
            var collideAll3 = CollideAll<Solid>();
            if (collideAll3.Count != 0 && collideAll3[0] != null)
            {
                velocity.X *= 0.85F;
            }
            velocity.X *= 0.95F;
            velocity.Y *= 0.15F;

            this.Position += velocity;
        }

        public override void Render()
        {
            base.Render();
        }

        public override void Added(Scene scene)
        {
            base.Added(scene);
            sprite = spriteBank.Create("minecart");
            sprite.Play("idle");
            Add(sprite);
        }

        public static void LoadContent()
        {
            spriteBank = new SpriteBank(GFX.Game, "Graphics/Mine/Sprites.xml");
        }
    }
}