﻿using Celeste;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using System.Collections;

using System.Collections.Generic;
using Celeste.Mod.Entities;

using On.Celeste;
using System.Linq;
using Celeste.Mod.Mine.Entities;
using Microsoft.Xna.Framework.Graphics;
using IL.Celeste;

namespace Celeste.Mod.Mine.Entities
{
    [Tracked(false)]
    [CustomEntity("Mine/TNT")]
    public class TNT : Actor
    {
        private static SpriteBank spriteBank;
        private Sprite sprite;
        private float range = 0;
        private float SpriteY = 0;
        public Holdable Hold;
        private Vector2 originPosition;
        private VertexLight light;

        public TNT(Vector2 position)
            : base(position)
        {
            base.Depth = -100;
            originPosition = position;
            base.Collider = new Hitbox(8f, 16, -4, -8);
            // base.Collider = new Hitbox(8f, 10f, -4f, -10f);
            Add(Hold = new Holdable(0.3f));
            Hold.PickupCollider = new Hitbox(16, 16, -8, 0);
            Hold.SlowFall = false;
            Hold.SlowRun = false;
            Hold.OnPickup = OnPickup;
            Hold.OnRelease = OnRelease;
            Hold.OnHitSpring = HitSpring;
            PlayPos = new Vector2(0);
            onCollideH = OnCollideHF;
            onCollideV = OnCollideVF;
            Hold.SpeedGetter = (() => Speed);

            LiftSpeedGraceTime = 0.1f; ;
            Speed = new Vector2(0);
            light = new VertexLight(Color.Red, 1, 10, 15);
            sprite = spriteBank.Create("TNT");
            sprite.Play("idle");
            sprite.CenterOrigin();

            Add(sprite);
            light.EndRadius = 0;
            light.StartRadius = 0;
            Add(light);
            Level = Scene as Level;
        }

        public TNT(EntityData e, Vector2 position)
            : this(e.Position + position)

        {
        }

        public Vector2 Speed;
        private bool first = false;
        private bool dead = false;
        public float explosionCoolDown = 5f;
        public bool exploding = false;
        private Level Level;
        private bool boom = false;
        private Collision onCollideH;
        private const float radius = 6;
        private Collision onCollideV;
        private float ending = 1;
        private float noGravityTimer;

        private Vector2 prevLiftSpeed;
        private Vector2 PlayPos;

        private BloomPoint bloom = new BloomPoint(0f, 32f);
        public static ParticleType P_Active;
        public static ParticleType P_Boom;
        private float swatTimer;

        private bool shattering;

        private float hardVerticalHitSoundCooldown = 0;

        public override void Render()
        {
            Level l = Scene as Level;
            //foreach (Entity item in l.Entities)
            //{
            //    if (item.Components.Get<Holdable>() != null)
            //    {
            //        Holdable h = item.Components.Get<Holdable>();
            //        if (h.IsHeld)
            //        {
            //            item.Position.Y = h.Holder.Position.Y - (item.Height * 2);
            //        }
            //    }
            //}
            sprite.DrawOutline();
            base.Render();
            if (l.BetweenInterval(1f) && exploding)
            {
                Draw.Rect(new Vector2(sprite.Position.X - sprite.Width / 2 + this.Position.X, sprite.Position.Y - sprite.Height / 2 + this.Position.Y), sprite.Width, sprite.Height, Color.White);
                // sprite.DrawOutline(Color.White);
            }
            if (this.exploding == true && ending == 1)
            {
                float pAngle = (PlayPos - this.Position).Angle();
                int quality = 28;
                for (int i = 0; i < quality * 2; i++)
                {
                    float rad2 = Calc.Map((float)i / quality, 0f, 1f, -(float)Math.PI / 30f, 3.24631262f);
                    Vector2 angle = Calc.AngleToVector(rad2, 1f);
                    Vector2 pos = Position + angle * (radius * 8);
                    Draw.Point(pos, Color.White);
                }
                //Draw.Circle(this.Center, 5 * 8, Color.Red, 100);
                if (ending != 1)
                {
                    //Draw.SpriteBatch.(this.Center, (5 * 8) * (1 - ending), Color.White, 50);
                }
            }
            if (exploding)
            {
                Draw.Circle(this.Position, (radius * 8) / 5 * (5 - explosionCoolDown), new Color(1f, 1f, 1f, 0.1f), 10);
            }
        }

        private void Boom()
        {
            Level l = Scene as Level;

            if (ending != 1)
            {
                light.EndRadius = 0;
                light.StartRadius = 0;
                bool shake = Settings.Instance.DisableScreenShake;
                Settings.Instance.DisableScreenShake = false;
                l.Shake(0.3f);
                Settings.Instance.DisableScreenShake = shake;
                l.Displacement.AddBurst(Position, 0.2f, 16f, 128f, 1f);
                Circle circ = new Circle(radius * 8, this.Center.X, this.Center.Y);
                for (int i = 0; i < 5; i++)
                {
                    l.ParticlesFG.Emit(P_Boom, this.Position);
                }

                foreach (CrackedStone item in l.Entities.FindAll<CrackedStone>())
                {
                    if (item.Collider.Collide(circ))
                    {
                        l.ParticlesFG.Emit(P_Boom, item.Center);
                        item.removeBlock();
                    }
                }
                Player player = l.Entities.FindFirst<Player>();
                if (player != null)
                {
                    PlayPos = player.Position;

                    if (player.Collider.Collide(circ))
                    {
                        player.ExplodeLaunch(new Vector2(this.X, this.Y), true);
                        player.Speed.X = (int)(player.Speed.X * 1.1F);
                        player.Speed.Y = (int)(player.Speed.Y * 1.1F);
                        ending = -1;
                    }
                }
                List<TouchSwitch> switchs = new List<TouchSwitch>();
                switchs = l.Entities.FindAll<TouchSwitch>();
                if (switchs.Count != 0)
                {
                    foreach (TouchSwitch item in switchs)
                    {
                        if (item.Collider.Collide(circ) || item.Collider.Collide(this))
                        {
                            item.TurnOn();
                        }
                    }
                }
            }
            ending -= Engine.DeltaTime * 5;

            if (ending < 0)
            {
                this.RemoveSelf();
            }
        }

        public override void Update()
        {
            Level l = Scene as Level;
            //if (first == false)
            //{
            //    first = true;
            //foreach (Entity item in l.Entities)
            //{
            //    if (item is SolidTiles)
            //    {
            //        continue;
            //    }
            //    Holdable h = new Holdable();
            //    Collider collider;
            //    if (item.Collider == null)
            //    {
            //        collider = new Hitbox(item.Width, item.Height);
            //    }
            //    else
            //    {
            //        collider = item.Collider;
            //    }
            //    h.PickupCollider = new Hitbox(collider.Width + 2, 2, collider.Position.X - 1, collider.Position.Y + collider.Height);

            // h.SlowFall = false; h.SlowRun = false;

            //    item.Add(h);
            //}
            //}

            sprite.Y = SpriteY;
            base.Update();
            if (shattering || dead)
            {
                return;
            }

            if (swatTimer > 0f)
            {
                swatTimer -= Engine.DeltaTime;
            }
            hardVerticalHitSoundCooldown -= Engine.DeltaTime;

            if (Hold.IsHeld && range < 1)
            {
                range += Engine.DeltaTime * 3;
            }
            else if (range > 0)
            {
                range -= Engine.DeltaTime * 3;
            }

            SpriteY = Vector2.Lerp(new Vector2(0, 0), new Vector2(0, -8), Ease.SineInOut(range)).Y;
            base.Depth = 100;
            if (Hold.IsHeld)
            {
                prevLiftSpeed = Vector2.Zero;
            }
            else
            {
                if (OnGround())
                {
                    float targetX = (!OnGround(Position + Vector2.UnitX * 3f)) ? 20f : (OnGround(Position - Vector2.UnitX * 3f) ? 0f : (-20f));
                    Speed.X = Calc.Approach(Speed.X, targetX, 800f * Engine.DeltaTime);
                    Vector2 lift = base.LiftSpeed;
                    if (lift == Vector2.Zero && prevLiftSpeed != Vector2.Zero)
                    {
                        Speed = prevLiftSpeed;
                        prevLiftSpeed = Vector2.Zero;
                        Speed.Y = Math.Min(Speed.Y * 0.6f, 0f);
                        if (Speed.X != 0f && Speed.Y == 0f)
                        {
                            Speed.Y = -60f;
                        }
                        if (Speed.Y < 0f)
                        {
                            noGravityTimer = 0.15f;
                        }
                    }
                    else
                    {
                        prevLiftSpeed = lift;
                        if (lift.Y < 0f && Speed.Y < 0f)
                        {
                            Speed.Y = 0f;
                        }
                    }
                }
                else if (Hold.ShouldHaveGravity)
                {
                    float grav = 800f;
                    if (Math.Abs(Speed.Y) <= 30f)
                    {
                        grav *= 0.5f;
                    }
                    float fric = 350f;
                    if (Speed.Y < 0f)
                    {
                        fric *= 0.5f;
                    }
                    Speed.X = Calc.Approach(Speed.X, 0f, fric * Engine.DeltaTime);
                    if (noGravityTimer > 0f)
                    {
                        noGravityTimer -= Engine.DeltaTime;
                    }
                    else
                    {
                        Speed.Y = Calc.Approach(Speed.Y, 200f, grav * Engine.DeltaTime);
                    }
                }
                MoveH(Speed.X * Engine.DeltaTime, OnCollideHF);
                MoveV(Speed.Y * Engine.DeltaTime, OnCollideVF);
                //if (base.Center.X > (float)Level.Bounds.Right)
                //{
                //    MoveH(32f * Engine.DeltaTime);
                //    if (base.Left - 8f > (float)Level.Bounds.Right)
                //    {
                //        RemoveSelf();
                //    }
                //}
                //else if (base.Left < (float)Level.Bounds.Left)
                //{
                //    base.Left = Level.Bounds.Left;
                //    Speed.X *= -0.4f;
                //}
                //else if (base.Top < (float)(Level.Bounds.Top - 4))
                //{
                //    base.Top = Level.Bounds.Top + 4;
                //    Speed.Y = 0f;
                //}
                //else if (base.Bottom > (float)Level.Bounds.Bottom && SaveData.Instance.Assists.Invincible)
                //{
                //    base.Bottom = Level.Bounds.Bottom;
                //    Speed.Y = -300f;
                //    Audio.Play("event:/game/general/assist_screenbottom", Position);
                //}
                //else if (base.Top > (float)Level.Bounds.Bottom)
                //{
                //    Die();
                //}
                //if (base.X < (float)(Level.Bounds.Left + 10))
                //{
                //    MoveH(32f * Engine.DeltaTime);
                //}
            }
            if (!dead)
            {
                Hold.CheckAgainstColliders();
            }
            if (this.CollideCheck<LazerBeam>() && exploding == false)
            {
                exploding = true;
                explosionCoolDown = -0.1f;
            }
            if (exploding == true)
            {
                light.Alpha = 1;
                explosionCoolDown -= Engine.DeltaTime * 2;
                if (l.BetweenInterval(0.1f))
                {
                    l.ParticlesBG.Emit(P_Active, this.Position + new Vector2(8, 0), (270f).ToRad());
                }
                if (0 > explosionCoolDown)
                {
                    Boom();
                    this.Remove(sprite);
                }
            }
        }

        public IEnumerator Shatter()
        {
            shattering = true;
            BloomPoint bloom = new BloomPoint(0f, 32f);
            VertexLight light = new VertexLight(Color.AliceBlue, 0f, 64, 200);
            Add(bloom);
            Add(light);
            for (float p = 0f; p < 1f; p += Engine.DeltaTime)
            {
                Position += Speed * (1f - p) * Engine.DeltaTime;
                Level.ZoomFocusPoint = base.TopCenter - Level.Camera.Position;
                light.Alpha = p;
                bloom.Alpha = p;
                yield return null;
            }
            yield return 0.5f;
            bool shake =
            Settings.Instance.DisableScreenShake;
            Settings.Instance.DisableScreenShake = false;
            Level.Shake();
            sprite.Play("shatter");
            yield return 1f;
            Level.Shake();
            yield return 0.3f;
            Settings.Instance.DisableScreenShake = shake
            ;
        }

        public void ExplodeLaunch(Vector2 from)
        {
            if (!Hold.IsHeld)
            {
                Speed = (base.Center - from).SafeNormalize(120f);
                SlashFx.Burst(base.Center, Speed.Angle());
            }
        }

        public bool HitSpring(Spring spring)
        {
            if (!Hold.IsHeld)
            {
                if (spring.Orientation == Spring.Orientations.Floor && Speed.Y >= 0f)
                {
                    Speed.X *= 0.5f;
                    Speed.Y = -160f;
                    noGravityTimer = 0.15f;
                    return true;
                }
                if (spring.Orientation == Spring.Orientations.WallLeft && Speed.X <= 0f)
                {
                    MoveTowardsY(spring.CenterY + 5f, 4f);
                    Speed.X = 220f;
                    Speed.Y = -80f;
                    noGravityTimer = 0.1f;
                    return true;
                }
                if (spring.Orientation == Spring.Orientations.WallRight && Speed.X >= 0f)
                {
                    MoveTowardsY(spring.CenterY + 5f, 4f);
                    Speed.X = -220f;
                    Speed.Y = -80f;
                    noGravityTimer = 0.1f;
                    return true;
                }
            }
            return false;
        }

        private void OnCollideHF(CollisionData data)
        {
            if (Math.Abs(Speed.X) > 100f)
            {
                ImpactParticles(data.Direction);
            }
            Speed.X *= -0.4f;
        }

        private void OnCollideVF(CollisionData data)
        {
            if (Speed.Y > 160f)
            {
                ImpactParticles(data.Direction);
            }
            if (Speed.Y > 140f)
            {
                Speed.Y *= -0.6f;
            }
            else
            {
                Speed.Y = 0f;
            }
        }

        private void ImpactParticles(Vector2 dir)
        {
            float angle = 0;
            Vector2 at;
            Vector2 range;
            if (dir.X > 0f)
            {
                angle = (float)Math.PI;
                at = new Vector2(base.Right, base.Y - 4f);
                range = Vector2.UnitY * 6f;
            }
            else if (dir.X < 0f)
            {
                angle = 0f;
                at = new Vector2(base.Left, base.Y - 4f);
                range = Vector2.UnitY * 6f;
            }
            else if (dir.Y > 0f)
            {
                angle = -(float)Math.PI / 2f;
                at = new Vector2(base.X, base.Bottom);
                range = Vector2.UnitX * 6f;
            }
            else
            {
                angle = (float)Math.PI / 2f;
                at = new Vector2(base.X, base.Top);
                range = Vector2.UnitX * 6f;
            }
        }

        public override bool IsRiding(Solid solid)
        {
            if (Speed.Y == 0f)
            {
                return base.IsRiding(solid);
            }
            return false;
        }

        protected override void OnSquish(CollisionData data)
        {
            if (!TrySquishWiggle(data) && !SaveData.Instance.Assists.Invincible)
            {
                this.Position = originPosition;
            }
        }

        private void OnPickup()
        {
            Speed = Vector2.Zero;
            AddTag(Tags.Persistent);
        }

        private void OnRelease(Vector2 force)
        {
            exploding = true;
            Add(bloom);

            light.EndRadius = 20;
            light.StartRadius = 10;
            RemoveTag(Tags.Persistent);
            if (force.X != 0f && force.Y == 0f)
            {
                force.Y = -0.4f;
            }
            Speed = force * 200f;
            if (Speed != Vector2.Zero)
            {
                noGravityTimer = 0.1f;
            }
        }

        public void Die()
        {
            if (!dead)
            {
                this.RemoveSelf();
            }
        }

        public override void Added(Scene scene)
        {
            base.Added(scene);
        }

        public static void LoadContent()
        {
            spriteBank = new SpriteBank(GFX.Game, "Graphics/Mine/Sprites.xml");
        }
    }
}