﻿using Celeste.Mod.Entities;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using System.Collections.Generic;

namespace Celeste.Mod.Mine.Entities
{
    [Tracked(false)]
    [CustomEntity("Mine/UpPlatform")]
    public class UpPlatform : Solid
    {
        private int updt = 0;
        private static SpriteBank spriteBank;
        private Sprite sprite;
        public static ParticleType upwardParticle;

        public UpPlatform(Vector2 position)
            : base(new Vector2(position.X - 16, position.Y), 32, 3, true)
        {
        }

        public UpPlatform(EntityData e, Vector2 position)
            : this(e.Position + position)

        {
        }

        public override void Update()
        {
            base.Update();
            Level lev = Scene as Level;
            updt += 1;
            if (updt % 2 == 0)
            {
                this.MoveV(-1f);
            }
            if (this.Y < lev.Camera.Y)
            {
                this.RemoveSelf();
            }
            if (lev.BetweenInterval(0.03f))
            {
                Vector2 particlePos = new Vector2(this.X + Calc.Random.Range(-16, 16) + 16 + 8, this.Y + 8);
                lev.ParticlesBG.Emit(upwardParticle, particlePos, (90f).ToRad());
            }
        }

        public override void Render()
        {
            base.Render();
        }

        public override void Added(Scene scene)
        {
            base.Added(scene);
            sprite = spriteBank.Create("UpPlatform");
            sprite.Play("idle");
            Add(sprite);
        }

        public static void LoadContent()
        {
            spriteBank = new SpriteBank(GFX.Game, "Graphics/Mine/Sprites.xml");
        }
    }

    [Tracked(false)]
    [CustomEntity("Mine/UpPlatformSpawn")]
    public class UpPlatformSpawn : Entity
    {
        private bool alreadySpawn = false;

        public UpPlatformSpawn(Vector2 position)
            : base(position)
        {
        }

        public UpPlatformSpawn(EntityData e, Vector2 position)
            : this(e.Position + position)

        {
        }

        public override void Update()
        {
            base.Update();
            Level l = Scene as Level;
            if (l.BetweenInterval(2.3f))
            {
                if (alreadySpawn == false)
                {
                    l.Add(new UpPlatform(this.Position));
                    alreadySpawn = true;
                }
            }
            else
            {
                alreadySpawn = false;
            }
        }

        public override void Render()
        {
            Level l = Scene as Level;
            Draw.Line(this.Position, new Vector2(this.Position.X, l.Camera.Y), Color.Gray);
            base.Render();
        }

        public override void Added(Scene scene)
        {
            base.Added(scene);
        }

        public static void LoadContent()
        {
        }
    }
}