﻿using Celeste.Mod;
using FMOD.Studio;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using On.Celeste;
using Mono.Cecil.Cil;
using Celeste.Mod.Mine.Entities;
using Celeste.Mod.Mine.Cutscene;
using Mono.Cecil;

namespace Celeste.Mod.Mine

{
    public class Mine : EverestModule
    {
        public static Mine Instance;
        public SpriteBank spriteBank;
        public override Type SettingsType => typeof(MineModuleSettings);
        public static MineModuleSettings Settings => (MineModuleSettings)Instance._Settings;
        private bool debug = true;

        static public VirtualMap<char> map;

        public Mine()
            : base()
        {
            Instance = this;

            // Metadata.Name = "The Mine";
        }

        public override void LoadSettings()
        {
            base.LoadSettings();
        }

        public override void Load()
        {
            // This is a runtime mod, but we can still manipulate the game.
        }

        public override void LoadContent(bool firstLoad)
        {
            Logger.Log("loading (TheMine mod)", "start loading content...");
            Logger.Log("loading (TheMine mod)", "start loading hook function...");
            On.Celeste.BloomRenderer.Apply += BloomRenderer_Apply; ;
            // On.Celeste.Level.Update += Level_Update; On.Celeste.Level.Render += ;
            Logger.Log("loading (TheMine mod)", "start loading ressource...");
            LazerBeam.LoadContent();
            Chain.LoadContent();
            CrystalBall.LoadContent();
            CrystalShooter.LoadContent();
            Rail.LoadContent();
            Minecart.LoadContent();
            TeacherGravestone.LoadContent();
            JumpBar.LoadContent();
            DarkManager.LoadContent();
            MagicLight.LoadContent();
            MoveMagicLight.LoadContent();
            MineGlider.LoadContent();
            UpPlatform.LoadContent();
            TNT.LoadContent();
            CrackedStone.LoadContent();
            FireManager.LoadContent();
            spriteBank = new SpriteBank(GFX.Game, "Graphics/Mine/Sprites.xml");

            Logger.Log("loading (TheMine mod)", "start loading particle...");
            CrystalBall.P_Trail = new ParticleType
            {
                Size = 2f,
                Color = Calc.HexToColor("6879A0"),
                Color2 = Calc.HexToColor("000029"),

                ColorMode = ParticleType.ColorModes.Fade,
                FadeMode = ParticleType.FadeModes.Late,
                SpeedMin = 10f,
                SpeedMax = 30f,
                DirectionRange = 1f,
                LifeMin = 0.2f,
                LifeMax = 0.5f
            }; DashZone.P_Zone = new ParticleType
            {
                Size = 1.1f,
                Color = Color.FromNonPremultiplied(255, 255, 255, 30),
                Color2 = Color.FromNonPremultiplied(255, 255, 255, 0),

                ColorMode = ParticleType.ColorModes.Fade,
                FadeMode = ParticleType.FadeModes.Late,
                SpeedMin = 5f,
                SpeedMax = 10f,
                DirectionRange = 0.2f,
                LifeMin = 1.2f,
                LifeMax = 2.5f
            }; DashZone.P_Player = new ParticleType
            {
                Size = 1.1f,
                Color = Color.FromNonPremultiplied(255, 255, 255, 100),
                Color2 = Color.FromNonPremultiplied(255, 255, 255, 0),
                ScaleOut = true,
                ColorMode = ParticleType.ColorModes.Fade,
                FadeMode = ParticleType.FadeModes.Late,
                SpeedMin = 5f,
                SpeedMax = 10f,
                DirectionRange = 2f,
                LifeMin = 1.2f,
                LifeMax = 2.5f
            };
            ForcedDashzone.P_Zone = new ParticleType
            {
                Size = 1.1f,
                Color = Color.FromNonPremultiplied(255, 200, 200, 30),
                Color2 = Color.FromNonPremultiplied(255, 200, 200, 0),

                ColorMode = ParticleType.ColorModes.Fade,
                FadeMode = ParticleType.FadeModes.Late,
                SpeedMin = 5f,
                SpeedMax = 10f,
                DirectionRange = 0.2f,
                LifeMin = 1.2f,
                LifeMax = 2.5f
            }; ForcedDashzone.P_Player = new ParticleType
            {
                Size = 1.1f,
                Color = Color.FromNonPremultiplied(255, 200, 200, 100),
                Color2 = Color.FromNonPremultiplied(255, 200, 200, 0),
                ScaleOut = true,
                ColorMode = ParticleType.ColorModes.Fade,
                FadeMode = ParticleType.FadeModes.Late,
                SpeedMin = 5f,
                SpeedMax = 10f,
                DirectionRange = 2f,
                LifeMin = 1.2f,
                LifeMax = 2.5f
            }; JumpBar.P_Activate = new ParticleType
            {
                Size = 2f,
                Color = Color.FromNonPremultiplied(100, 255, 100, 255),
                Color2 = Color.FromNonPremultiplied(0, 0, 0, 0),
                ScaleOut = true,
                ColorMode = ParticleType.ColorModes.Fade,
                FadeMode = ParticleType.FadeModes.Late,
                SpeedMin = 20f,
                SpeedMax = 50f,
                DirectionRange = 0f,
                LifeMin = 1.2f,
                LifeMax = 5.5f
            }; JumpBar.P_Down = new ParticleType
            {
                Size = 1.1f,
                Color = Color.FromNonPremultiplied(255, 100, 100, 255),
                Color2 = Color.FromNonPremultiplied(0, 0, 0, 255),
                ScaleOut = true,
                ColorMode = ParticleType.ColorModes.Fade,
                FadeMode = ParticleType.FadeModes.Late,
                SpeedMin = 5f,
                SpeedMax = 10f,
                DirectionRange = 0f,
                LifeMin = 1.2f,
                LifeMax = 2.5f
            }; DarkManager.P_Dark = new ParticleType
            {
                Size = 2.1f,
                Color = Color.FromNonPremultiplied(0, 0, 0, 255),
                Color2 = Color.FromNonPremultiplied(0, 0, 0, 100),
                ScaleOut = false,
                ColorMode = ParticleType.ColorModes.Fade,
                FadeMode = ParticleType.FadeModes.Late,
                SpeedMin = 1f,
                SpeedMax = 10f,
                DirectionRange = 0f,
                LifeMin = 30.2f,
                LifeMax = 50.5f
            };
            MineGlider.P_Glide = new ParticleType
            {
                Acceleration = Vector2.UnitY * 60f,
                SpeedMin = 30f,
                SpeedMax = 40f,
                Direction = -(float)Math.PI / 2f,
                DirectionRange = (float)Math.PI / 2f,
                LifeMin = 0.6f,
                LifeMax = 1.2f,
                ColorMode = ParticleType.ColorModes.Blink,
                FadeMode = ParticleType.FadeModes.Late,
                Color = Calc.HexToColor("858786"),
                Color2 = Calc.HexToColor("797A86"),
                Source = GFX.Game["particles/rect"],
                Size = 0.5f,
                SizeRange = 0.2f,
                RotationMode = ParticleType.RotationModes.SameAsDirection
            };
            MineGlider.P_GlideUp = new ParticleType(Glider.P_Glide)
            {
                Acceleration = Vector2.UnitY * -10f,
                SpeedMin = 50f,
                SpeedMax = 60f
            };
            MineGlider.P_Platform = new ParticleType
            {
                Acceleration = Vector2.UnitY * 60f,
                SpeedMin = 5f,
                SpeedMax = 20f,
                Direction = -(float)Math.PI / 2f,
                LifeMin = 0.6f,
                LifeMax = 1.4f,
                FadeMode = ParticleType.FadeModes.Late,
                Size = 1f
            };
            MineGlider.P_Glow = new ParticleType
            {
                SpeedMin = 8f,
                SpeedMax = 16f,
                DirectionRange = (float)Math.PI * 2f,
                LifeMin = 0.4f,
                LifeMax = 0.8f,
                Size = 1f,
                FadeMode = ParticleType.FadeModes.Late,
                Color = Calc.HexToColor("6D747A"),
                Color2 = Calc.HexToColor("F2FDFF"),
                ColorMode = ParticleType.ColorModes.Blink
            };
            MineGlider.P_Expand = new ParticleType(Glider.P_Glow)
            {
                SpeedMin = 40f,
                SpeedMax = 80f,
                SpeedMultiplier = 0.2f,
                LifeMin = 0.6f,
                LifeMax = 1.2f,
                DirectionRange = (float)Math.PI * 3f / 4f
            };
            UpPlatform.upwardParticle = new ParticleType
            {
                Size = 1f,
                SizeRange = 0,
                SpeedMax = 10f,
                SpeedMin = 5f,
                DirectionRange = 0,
                LifeMin = 0.4f,
                LifeMax = 1.2f,
                Source = GFX.Game["particles/starfield/00"],
                FadeMode = ParticleType.FadeModes.Late,
                Color = Calc.HexToColor("898989"),
                Color2 = Calc.HexToColor("212121"),
                ColorMode = ParticleType.ColorModes.Fade
            };
            TNT.P_Active = new ParticleType
            {
                Size = 1f,
                SizeRange = 0,
                SpeedMax = 20f,
                SpeedMin = 5f,
                DirectionRange = 0.5f,
                LifeMin = 0.4f,
                LifeMax = 7.2f,
                Source = GFX.Game["particles/starfield/00"],
                FadeMode = ParticleType.FadeModes.Late,
                Color = Calc.HexToColor("898989"),
                Color2 = Calc.HexToColor("212121"),
                ColorMode = ParticleType.ColorModes.Fade
            };
            TNT.P_Boom = new ParticleType
            {
                Size = 0.5f,
                SizeRange = 0.4f,
                SpeedMax = 7f,
                SpeedMin = 3f,
                DirectionRange = 360f,
                LifeMin = 1.4f,
                LifeMax = 2.2f,
                Source = GFX.Game["particles/tntCircle"],
                FadeMode = ParticleType.FadeModes.Late,
                Color = Calc.HexToColor("898989"),
                Color2 = Calc.HexToColor("212121"),
                ColorMode = ParticleType.ColorModes.Fade
            };
            FireManager.P_Fire = new ParticleType
            {
                Size = 1f,
                SizeRange = 0f,
                SpeedMax = 60f,
                SpeedMin = 3f,
                Direction = (-90f).ToRad(),
                DirectionRange = 1f,
                LifeMin = 1.4f,
                LifeMax = 20.2f,
                Source = GFX.Game["particles/starfield/00"],
                FadeMode = ParticleType.FadeModes.Late,
                Color = Calc.HexToColor("DA2E21"),
                Color2 = Calc.HexToColor("FB8521"),

                ColorMode = ParticleType.ColorModes.Fade
            };
            Logger.Log("loading (TheMine mod)", "load content finish !");
        }

        private void BloomRenderer_Apply(On.Celeste.BloomRenderer.orig_Apply orig, BloomRenderer self, VirtualRenderTarget target, Scene scene)
        {
            if (self != null)
            {
                Level l = scene as Level;
                if (l.Entities != null)
                {
                    DarkManager DM = l.Entities.FindFirst<DarkManager>();
                    if (DM != null)
                    {
                        DM.DrawDark(target);
                    }
                    else
                    {
                    }
                }
                if (l.Entities != null)
                {
                    FireManager DM = l.Entities.FindFirst<FireManager>();
                    if (DM != null)
                    {
                        DM.DrawData(target);
                    }
                    else
                    {
                    }
                }
            }
            orig(self, target, scene);
        }

        private void LightingRenderer_Render(On.Celeste.LightingRenderer.orig_Render orig, LightingRenderer self, Scene scene)
        {
        }

        public void Level_AfterRender(On.Celeste.Level.orig_Render orig, Level self)
        {
        }

        public override void Unload()
        {
        }

        public override void CreateModMenuSection(TextMenu menu, bool inGame, EventInstance snapshot)
        {
            base.CreateModMenuSection(menu, inGame, snapshot);
        }
    }
}